
#
# Shiny App demonstrating the top-k interface of rPref
# ****************************************************
#
# Server File
#
# Press "Run App" in RStudio to start the App
#


library(shiny)
library(rPref)

shinyServer(function(input, output, session) {
  

  # generate the plot
  output$topkPlot <- renderPlot({
    
    # Initialize
    # ----------
    
    # the Skyline: low fuel consumption (high mpg) and high power
    pref <- high(mpg) * high(hp)
  
    # collect arguments for the preference selection
    args <- list()
    
    # collect explain-string
    explain <- character(0)
    
    # Add TOP-k options to args and explain
    topk_options <- 0
    if (input$topk > 0) { # top-k
      topk_options <- topk_options + 1
      args$top <- input$topk
      explain <- c(explain, paste0("exactly ", input$topk, " tuples"))
    }
    if (input$atleast > 0) { # at least
      topk_options <- topk_options + 1
      args$at_least  <- input$atleast
      explain <- c(explain, paste0("at least ", input$atleast, " tuples and
                                   fill with those having the lowest level of them"))
    }
    if (input$toplevel > 0) { # top level
      topk_options <- topk_options + 1
      args$top_level <- input$toplevel
      explain <- c(explain, paste0("all tuples with a level of ", input$toplevel))
    }
    
    # Add intersection/union parameter for top-k selectin
    top_connect_select <- NULL
    if(topk_options > 1) {
      args$and_connected <- TRUE # default
      if (!is.null(input$topk_connect)) {
        # for re-creating the control with correct selected-value
        top_connect_select <- input$topk_connect 
        if (input$topk_connect == "OR") args$and_connected <- FALSE
      }
    }
    
    # Convert top-k argument to string (for raw command displayed in UI
    if (length(args) > 0)
      topk_args_string <- paste0(", ", paste(paste0(names(args), " = ", args), collapse = ", "))
    else
      topk_args_string <- ""
    
    # Add arguments for dataset and preference
    args <- c(args, list(df = mtcars, pref = pref))
    
    # Calculate the result
    # --------------------
    
    res <- do.call(psel, args)
    
    # Retrieve the level values
    if (topk_args_string == '') {
      un_levels <- 1
      level_vals <- rep(1, nrow(res))
    } else {
      un_levels <- unique(res$.level)
      level_vals <- res$.level
    }
    
    # Plot the graph
    # --------------
    
    plot(mtcars$mpg, mtcars$hp)
    plot_front(mtcars, pref)
    points(res$mpg, res$hp, lwd = 5, col = level_vals)
    legend(33, 330, xjust = 1, paste0("level ", un_levels), 
           lwd = 3, pch = 1, lty = 0, col = un_levels)
    
    
    # Update UI
    # ---------
    
    # top-k text feedback
    if (topk_options == 0) { # No top-k option chosen
      
      explain <- "Select Pareto-Opima (without top-k)"
      output$state <- renderText({"<span style='color:red'>Activate one of the top-k options
        (top-k, at-least, or top-level)!</span>"})
    
    } else if (topk_options == 1) { # exactly one top-k option
      
      explain <-  paste0("Select ", explain)
      output$state <- renderText({"<span style='color:green'>One top-k option selected.</span>"})
      
    } else if (topk_options > 1) { # more than one option => explain usage of intersection/union
      
      explain_start <- if (args$and_connected) "Select intersection of: "
                       else                    "Select union of: "
      explain <- paste0(explain_start, paste(explain, collapse = ", "))
      output$state <- renderText({"<span style='color:red'>More then one top-k option selected.
        Choose connection operator!</span>"})
    }
    
    output$explanation <- renderText({explain})
    
    # selection control for connection operator (union/intersection)
    output$topk_connect_frame <- renderUI({
      if (topk_options > 1)
        selectInput("topk_connect", "Connection operator", c("AND", "OR"), 
                    selected = top_connect_select)
    })
   
    # string output the number of tuples
    output$number <- renderText({paste0(nrow(res), " tuples selected")})
    
    # start of psel-command, missing closing bracket
    cmd_start <- paste( sep = ", ")
    
    # show the raw command of the preference selection
    output$command <- renderText({paste0("R command:  psel(mtcars, high(hp) * high(mpg)",
                                         topk_args_string, ")")})
  })
})